<?php

namespace Drupal\telephone_plus\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\telephone_plus\TelephonePlusFormatter;
use Drupal\telephone_plus\TelephonePlusValidator;

/**
 * Plugin implementation of the 'telephone_plus_link' formatter.
 *
 * @FieldFormatter(
 *   id = "telephone_plus",
 *   label = @Translation("Telephone Plus"),
 *   description = @Translation("Formats telephone fields as plain text or link and with or without hcard markup."),
 *   field_types = {
 *     "telephone_plus_field"
 *   }
 * )
 */
class TelephonePlusFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $options = parent::defaultSettings();

    $options['vcard'] = TRUE;
    $options['link'] = TRUE;
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display number as a link'),
      '#default_value' => $this->getSetting('link'),
    ];

    $form['vcard'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable vCard/hCard support'),
      '#default_value' => $this->getSetting('vcard'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $link_enabled = $this->getSetting('link');
    $vcard_enabled = $this->getSetting('vcard');

    $summary[] = $this->t('Display as link: %enabled', ['%enabled' => ($link_enabled) ? 'Yes' : 'No']);
    $summary[] = $this->t('vCard enabled: %enabled', ['%enabled' => ($vcard_enabled) ? 'Yes' : 'No']);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $vcard = $this->getSetting('vcard') ?? FALSE;
    $as_link = $this->getSetting('link') ?? FALSE;

    foreach ($items as $delta => $item) {
      $telephone = new TelephonePlusValidator($item->telephone_number, $item->telephone_extension, $item->country_code);

      // If we don't have a valid number, set variables to allow fallback to
      // plain text.
      if (!$telephone->isValid()) {
        $telephone_text = $item->telephone_number;
        $telephone_link = '';
      }
      else {
        $telephone = new TelephonePlusFormatter($item->telephone_number, $item->telephone_extension, $item->country_code);
        // TelephonePlus link text.
        $telephone_link = $telephone->url();
        // TelephonePlus display text.
        $telephone_text = $telephone->text($item->display_international_number);
      }

      $attributes = ['class' => ['telephone-plus__item-number']];

      if ($vcard) {
        $attributes['class'][] = 'value';
      }

      if (!empty($telephone_link) && $as_link) {
        // Url::fromUri() doesn't place nice with the generated tel: URI
        // so reverting to using an anchor html_tag instead of a Drupal
        // link element.
        $attributes['href'] = $telephone_link;

        $telephone_number = [
          '#type' => 'html_tag',
          '#tag' => 'a',
          '#attributes' => $attributes,
          '#value' => $telephone_text,
        ];
      }
      else {
        $telephone_number = [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#attributes' => $attributes,
          '#value' => $telephone_text,
        ];
      }

      $elements[$delta] = [
        '#theme' => 'telephone_plus_item',
        '#title' => $item->telephone_title,
        '#number' => $telephone_number,
        '#extension' => $item->telephone_extension,
        '#supplementary' => $item->telephone_supplementary,
        '#vcard' => $vcard,
      ];
    }

    // If vCard option is enabled wrap all the elements with the required
    // class and include the microformat link.
    if ($this->getSetting('vcard')) {
      $elements['#attributes']['class'][] = 'vcard';
      $elements['#attached']['html_head_link'][] = [
        [
          'rel' => 'profile',
          'href' => 'http://microformats.org/profile/hcard',
        ],
      ];
    }

    return $elements;
  }

}
